/* THIS FILE IS AUTOGENERATED FROM ActivityRequestHandler.webidl BY Codegen.py - DO NOT EDIT */

#ifndef mozilla_dom_ActivityRequestHandlerBinding_h
#define mozilla_dom_ActivityRequestHandlerBinding_h

#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/CallbackInterface.h"
#include "mozilla/dom/Nullable.h"
#include "mozilla/dom/ToJSValue.h"
#include "nsWeakReference.h"

namespace mozilla {
namespace dom {

struct ActivityOptions;
class ActivityRequestHandler;
struct ActivityRequestHandlerAtoms;
struct NativePropertyHooks;
class ProtoAndIfaceCache;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

namespace ActivityRequestHandlerBinding {

  typedef mozilla::dom::ActivityRequestHandler NativeType;

  // We declare this as an array so that retrieving a pointer to this
  // binding's property hooks only requires compile/link-time resolvable
  // address arithmetic.  Declaring it as a pointer instead would require
  // doing a run-time load to fetch a pointer to this binding's property
  // hooks.  And then structures which embedded a pointer to this structure
  // would require a run-time load for proper initialization, which would
  // then induce static constructors.  Lots of static constructors.
  extern const NativePropertyHooks sNativePropertyHooks[];

  bool
  ConstructorEnabled(JSContext* aCx, JS::Handle<JSObject*> aObj);

  JSObject*
  DefineDOMInterface(JSContext* aCx, JS::Handle<JSObject*> aGlobal, JS::Handle<jsid> id, bool aDefineOnGlobal);

  const JSClass*
  GetJSClass();

  bool
  Wrap(JSContext* aCx, mozilla::dom::ActivityRequestHandler* aObject, nsWrapperCache* aCache, JS::Handle<JSObject*> aGivenProto, JS::MutableHandle<JSObject*> aReflector);

  template <class T>
  inline JSObject* Wrap(JSContext* aCx, T* aObject, JS::Handle<JSObject*> aGivenProto)
  {
    JS::Rooted<JSObject*> reflector(aCx);
    return Wrap(aCx, aObject, aObject, aGivenProto, &reflector) ? reflector.get() : nullptr;
  }

  void
  ClearCachedSourceValue(mozilla::dom::ActivityRequestHandler* aObject);

  void
  CreateInterfaceObjects(JSContext* aCx, JS::Handle<JSObject*> aGlobal, ProtoAndIfaceCache& aProtoAndIfaceCache, bool aDefineOnGlobal);

  JS::Handle<JSObject*>
  GetProtoObjectHandle(JSContext* aCx, JS::Handle<JSObject*> aGlobal);

  JS::Handle<JSObject*>
  GetConstructorObjectHandle(JSContext* aCx, JS::Handle<JSObject*> aGlobal, bool aDefineOnGlobal = true);

  JSObject*
  GetConstructorObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal);

} // namespace ActivityRequestHandlerBinding



class ActivityRequestHandlerJSImpl : public CallbackInterface
{
public:
  explicit inline ActivityRequestHandlerJSImpl(JSContext* aCx, JS::Handle<JSObject*> aCallback, nsIGlobalObject* aIncumbentGlobal)
    : CallbackInterface(aCx, aCallback, aIncumbentGlobal)
  {
  }

  explicit inline ActivityRequestHandlerJSImpl(JSContext* aCx, JS::Handle<JSObject*> aCallback, nsIGlobalObject* aIncumbentGlobal, const FastCallbackConstructor& )
    : CallbackInterface(aCx, aCallback, aIncumbentGlobal, FastCallbackConstructor())
  {
  }

  explicit inline ActivityRequestHandlerJSImpl(JS::Handle<JSObject*> aCallback, JS::Handle<JSObject*> aAsyncStack, nsIGlobalObject* aIncumbentGlobal)
    : CallbackInterface(aCallback, aAsyncStack, aIncumbentGlobal)
  {
  }

  void PostResult(JS::Handle<JS::Value> result, ErrorResult& aRv, JSCompartment* aCompartment = nullptr);

  void PostError(const nsAString& error, ErrorResult& aRv, JSCompartment* aCompartment = nullptr);

  void __Init(const nsAString& id, const ActivityOptions& options, const Optional<bool>& returnvalue, ErrorResult& aRv, JSCompartment* aCompartment = nullptr);

  inline bool
  operator==(const ActivityRequestHandlerJSImpl& aOther) const
  {
    return CallbackInterface::operator==(aOther);
  }

  void GetSource(ActivityOptions& aRetVal, ErrorResult& aRv, JSCompartment* aCompartment = nullptr);

private:
  static bool
  InitIds(JSContext* cx, ActivityRequestHandlerAtoms* atomsCache);
};


class ActivityRequestHandler final : public nsSupportsWeakReference,
                                     public nsWrapperCache
{
public:
  NS_DECL_CYCLE_COLLECTING_ISUPPORTS
  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS(ActivityRequestHandler)

private:
  RefPtr<ActivityRequestHandlerJSImpl> mImpl;
  nsCOMPtr<nsISupports> mParent;

public:
  ActivityRequestHandler(JS::Handle<JSObject*> aJSImplObject, nsIGlobalObject* aParent);

private:
  ~ActivityRequestHandler();

public:
  nsISupports* GetParentObject() const;

  virtual JSObject* WrapObject(JSContext* aCx, JS::Handle<JSObject*> aGivenProto) override;

  static already_AddRefed<ActivityRequestHandler> Constructor(const GlobalObject& global, JSContext* cx, const nsAString& id, const ActivityOptions& options, const Optional<bool>& returnvalue, ErrorResult& aRv);

  void PostResult(JS::Handle<JS::Value> result, ErrorResult& aRv, JSCompartment* aCompartment = nullptr);

  void PostError(const nsAString& error, ErrorResult& aRv, JSCompartment* aCompartment = nullptr);

  void GetSource(ActivityOptions& aRetVal, ErrorResult& aRv, JSCompartment* aCompartment = nullptr) const;

  static bool
  _Create(JSContext* cx, unsigned argc, JS::Value* vp);
};


} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_ActivityRequestHandlerBinding_h
