/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
#ifndef nsUnicharUtilCIID_h__
#define nsUnicharUtilCIID_h__


#include "nsISupports.h"
#include "nscore.h"

#define NS_UNICHARUTIL_CID \
{ 0xcc10c750, 0x9ec3, 0x11d2, \
  { 0xb3, 0xae, 0x0, 0x80, 0x5f, 0x8a, 0x66, 0x70 } }

#define NS_UNICHARUTIL_CONTRACTID "@mozilla.org/intl/unicharutil;1"

#define NS_UNICHARCATEGORY_CID \
{ 0x748a1132, 0x671a, 0x409a, \
  { 0x8d, 0x1d, 0xf1, 0xcd, 0xf6, 0xb3, 0xa6, 0xb4 } }

#define NS_UNICHARCATEGORY_CONTRACTID "@mozilla.org/intl/unicharcategory;1"

#endif
