#define MOZ_SOURCE_STAMP b44c72b85a800d9c6e719579d480bb2c3a87a753
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-beta
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-beta/rev/b44c72b85a800d9c6e719579d480bb2c3a87a753
