/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIAtom;

%{C++
#define NS_ATOMSERVICE_CID \
{ /* ed3db3fc-0168-4cab-8818-98f5475a490c */ \
    0xed3db3fc,                              \
    0x0168,                                  \
    0x4cab,                                  \
    {0x88, 0x18, 0x98, 0xf5, 0x47, 0x5a, 0x49, 0x0c} }

#define NS_ATOMSERVICE_CONTRACTID "@mozilla.org/atom-service;1"
%}

/*
 * Should this really be scriptable?  Using atoms from script or proxies
 * could be dangerous since double-wrapping could lead to loss of
 * pointer identity.
 */
 
[scriptable, uuid(9c1f50b9-f9eb-42d4-a8cb-2c7600aeb241)]
interface nsIAtomService : nsISupports {

  /**
   * Version of NS_Atomize that doesn't require linking against the
   * XPCOM library.  See nsIAtom.idl.
   */
  nsIAtom getAtom(in AString value);
};
