/* THIS FILE IS AUTOGENERATED FROM AudioChannel.webidl BY Codegen.py - DO NOT EDIT */

#ifndef mozilla_dom_AudioChannelBinding_h
#define mozilla_dom_AudioChannelBinding_h

#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/Nullable.h"

namespace mozilla {
namespace dom {

struct NativePropertyHooks;
class ProtoAndIfaceCache;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

enum class AudioChannel : uint32_t {
  Normal,
  Content,
  Notification,
  Alarm,
  Telephony,
  Ringer,
  Publicnotification,
  System,
  EndGuard_
};

namespace AudioChannelValues {
extern const EnumEntry strings[9];
} // namespace AudioChannelValues

bool
ToJSValue(JSContext* aCx, AudioChannel aArgument, JS::MutableHandle<JS::Value> aValue);


} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_AudioChannelBinding_h
